package com.spoc.utils;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.db4o.*;
import com.db4o.config.ConfigScope;

import com.spoc.model.*;


public class Db4oUtil {
	
	private static final Log log = LogFactory.getLog(Db4oUtil.class);
	
    public static String YAPFILENAME  = "";
    public static String YAPDIRECTORY = "";
    public static int PORT            = 0;
    public static String USER         = "";
    public static String PASSWORD     = ""; 
        
    private static final ThreadLocal<ObjectContainer> dbThreadLocal = new ThreadLocal<ObjectContainer>();
    private static ObjectServer objectServer;
    
    
    public static ObjectContainer getObjectContainer() {
    	
    	log.info("GENERIC : Getting Object Container");
    	
        ObjectContainer oc = (ObjectContainer) dbThreadLocal.get();
        
        if (oc == null || oc.ext().isClosed()) {
            oc = getObjectServer().openClient();
            dbThreadLocal.set(oc);
        }
        
        return oc;        
    }
    
    public static ObjectContainer getObjectContainer(String hostName, int port, String user, String password, String db4oFileName, String db4oDirectory) {
    	
    	YAPFILENAME  = db4oFileName;
    	YAPDIRECTORY = db4oDirectory;
    	PORT = port;
    	USER = user;
    	PASSWORD = password;
    	
    	log.info("START : Getting Object Container");
  
        ObjectContainer oc = (ObjectContainer) dbThreadLocal.get();        
        
        if (oc == null || oc.ext().isClosed()) {
        	log.info("Creation of new Object Container : " + YAPDIRECTORY + "/" + YAPFILENAME + " user=" + USER + " port=" + PORT);
            oc = getObjectServer().openClient();
            dbThreadLocal.set(oc);    
            log.info("COMPLETE : Successfully created a new Object Container.");
        } else {
        	log.info("COMPLETE : Using Object Container already in existence.");
        }
        
        return oc;        
    }    

    public static void closeObjectContainer() {
    	
        ObjectContainer oc = (ObjectContainer) dbThreadLocal.get();
        
        String ocId = null==oc?"null":oc.toString();        
        log.info("Closing the current Object Container ( " + ocId + ")");
        
        dbThreadLocal.set(null);
        
        if (oc != null) {        	
        	log.info("Closing the current Object Container ( " + oc.toString() + ") : Object Container is not NULL, closing.");
        	oc.close();
        } else {
        	log.info("Closing the current Object Container : Object Container is NULL.");
        }
        
    }

    public synchronized static ObjectServer getObjectServer() {
    	
    	log.info("Getting Object Server");
    	
        if (objectServer == null) {
        	log.info("Creation of new Object Server : " + YAPDIRECTORY + "/" + YAPFILENAME + " user=" + USER + " port=" + PORT);
            objectServer = getObjectServerForFilename(YAPFILENAME, PORT);
            objectServer.grantAccess(USER, PASSWORD);
        } else {
        	log.info("Using Object Server already in existence.");
        }
        
        return objectServer;
    }  

    public static ObjectServer getObjectServerForFilename(String yapfilename, int port) {
    	
    	log.info("Db4o Configurations.");
    	
        File parentDir = getDbDirectory();
        File dbfile = new File(parentDir, yapfilename);

        // For replication purposes
        Db4o.configure().generateUUIDs(ConfigScope.GLOBALLY);
        Db4o.configure().generateVersionNumbers(ConfigScope.GLOBALLY);

        // Other options
        Db4o.configure().exceptionsOnNotStorable(true);
        Db4o.configure().objectClass("java.math.BigDecimal").translate(new com.db4o.config.TSerializable());

        // Open server
        log.info("Db4o.openServer(" + dbfile.getPath() + "," + port + ")");
        ObjectServer objectServer = Db4o.openServer(dbfile.getPath(), port);

        return objectServer;
    }

    private static File getDbDirectory() {

        String dbfile = System.getProperty("user.home") + YAPDIRECTORY;
        
        log.info("Looking in the following directory for the Db4o file: " + dbfile);
        
        File f = new File(dbfile);
        
        if (!f.exists()) {
            f.mkdirs();
        }
        
        return f;
    }
    
    public static void shutdown() {
    	
    	log.info("Shutting down Object Server.");
    	
        if (objectServer != null) {
            objectServer.close();
        }
    } 
   
    /*
     * The "stand alone" server is basically for quick backup use.
     * Only one user can use this db4o file.
     */
    public static ObjectServer getStandAloneObjectServer(String db4oFileName, int port) {
    	
        File parentDir = new File( System.getProperty("user.home") + YAPDIRECTORY );
        
        if (!parentDir.exists()) {
        	parentDir.mkdirs();
        }
     	        	        
        File dbfile = new File(parentDir, db4oFileName);

        return Db4o.openServer(dbfile.getPath(), port);
    	    	
    }
       
}
