package com.spoc.data;

import org.apache.commons.logging.*;
import org.springframework.beans.factory.*;
import org.springframework.util.ObjectUtils;

import com.spoc.utils.Db4oUtil;

import com.db4o.*;
import com.db4o.config.Configuration;


public class RemoteServerObjectContainer implements FactoryBean, InitializingBean, DisposableBean {

	private static final Log log = LogFactory.getLog(RemoteServerObjectContainer.class);

	private ObjectContainer container;	
	
	private Configuration configuration;	
	private String hostName;
	private int port;	
	private String user;
	private String password;	
	private String db4oFileName;
	private String db4oDirectory;
	

	public void setConfiguration(Configuration configuration) {
		this.configuration = configuration;
	}	
	
	public void setHostName(String hostName) {
		this.hostName = hostName;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public void setUser(String user) {
		this.user = user;
	}	
	
	public void setPassword(String password) {
		this.password = password;
	}	
	
	public void setDb4oFileName(String db4oFileName) {
		this.db4oFileName = db4oFileName;
	}
	
	public void setDb4oDirectory(String db4oDirectory) {
		this.db4oDirectory = db4oDirectory;
	}	
	

	/**
	 * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
	 */
	public void afterPropertiesSet() throws Exception {
		
		if (configuration == null) {
			configuration = Db4o.configure();
		}

		if ( (null == hostName) || (port <= 0) || (null == user) ) {
			
			log.error("Properties 'hostName', 'port', and/or 'user' are necessary to create the ObjectContainer.");
			throw new IllegalArgumentException("'hostName', 'port', and/or 'user' are necessary to create the ObjectContainer.");
			
		} else {
			
			log.info("Successfully opened the db4o remote, server-based ObjectContainer : " + ObjectUtils.getIdentityHexString(container));
			container = Db4oUtil.getObjectContainer(hostName, port, user, password, db4oFileName, db4oDirectory);	
			
		}

	}

	/**
	 * @see org.springframework.beans.factory.DisposableBean#destroy()
	 */
	public void destroy() throws Exception {
		log.info("Closing ObjectContainer : " + ObjectUtils.getIdentityHexString(container));
		Db4oUtil.closeObjectContainer();
	}
	
	/**
	 * @see org.springframework.beans.factory.FactoryBean#getObject()
	 */
	public Object getObject() throws Exception {
		return container;
	}

	/**
	 * @see org.springframework.beans.factory.FactoryBean#getObjectType()
	 */
	public Class getObjectType() {
		return (container != null ? container.getClass() : ObjectContainer.class);
	}

	/**
	 * @see org.springframework.beans.factory.FactoryBean#isSingleton()
	 */
	public boolean isSingleton() {
		return true;
	}	

}
